@@ -224,11 +224,11 @@ sm_stropen(fp, info, flags, rpool)
 
 #if SM_RPOOL
 	s = sm_rpool_malloc_x(rpool, sizeof(SM_STR_OBJ_T));
-#else /* SM_RPOOL */
+#else
 	s = sm_malloc(sizeof(SM_STR_OBJ_T));
 	if (s == NULL)
 		return -1;
-#endif /* SM_RPOOL */
+#endif
 
 	fp->f_cookie = s;
 	s->strio_rpool = rpool;
@@ -297,7 +297,7 @@ sm_strclose(fp)
 #if !SM_RPOOL
 	sm_free(s->strio_base);
 	s->strio_base = NULL;
-#endif /* !SM_RPOOL */
+#endif
 	return 0;
 }
 
