@@ -12,6 +12,7 @@
  */
 
 #include <sendmail.h>
+#include <sm/sendmail.h>
 
 SM_RCSID("@(#)$Id: envelope.c,v 8.313 2013-11-22 20:51:55 ca Exp $")
 
@@ -103,18 +104,9 @@ newenvelope(e, parent, rpool)
 	e->e_parent = parent;
 	assign_queueid(e);
 	e->e_ctime = curtime();
-#if _FFR_SESSID
-	e->e_sessid = e->e_id;
-#endif /* _FFR_SESSID */
 	if (parent != NULL)
 	{
 		e->e_msgpriority = parent->e_msgsize;
-#if _FFR_SESSID
-		if (parent->e_sessid != NULL)
-			e->e_sessid = sm_rpool_strdup_x(rpool,
-							parent->e_sessid);
-#endif /* _FFR_SESSID */
-
 		if (parent->e_quarmsg == NULL)
 		{
 			e->e_quarmsg = NULL;
@@ -135,7 +127,6 @@ newenvelope(e, parent, rpool)
 		(void) sm_io_flush(CurEnv->e_xfp, SM_TIME_DEFAULT);
 	if (sendmode != DM_NOTSET)
 		set_delivery_mode(sendmode, e);
-
 	return e;
 }
 
@@ -189,7 +180,7 @@ dropenvelope(e, fulldrop, split)
 
 	if (tTd(50, 1))
 	{
-		sm_dprintf("dropenvelope %p: id=", e);
+		sm_dprintf("dropenvelope %p: id=", (void *)e);
 		xputs(sm_debug_file(), e->e_id);
 		sm_dprintf(", flags=");
 		printenvflags(e);
@@ -264,7 +255,7 @@ dropenvelope(e, fulldrop, split)
 #if _FFR_PROXY
 		if (queueit && e->e_sendmode == SM_PROXY)
 			queueit = false;
-#endif /* _FFR_PROXY */
+#endif
 
 		/* see if a notification is needed */
 		if (bitset(QPINGONFAILURE, q->q_flags) &&
@@ -323,7 +314,7 @@ dropenvelope(e, fulldrop, split)
 
 			/* don't free, allocated from e_rpool */
 			e->e_message = sm_rpool_strdup_x(e->e_rpool, buf);
-			message(buf);
+			message("%s", buf);
 			e->e_flags |= EF_CLRQUEUE;
 		}
 		if (msg_timeout == MSG_NOT_BY)
@@ -370,8 +361,8 @@ dropenvelope(e, fulldrop, split)
 			    strcmp(e->e_from.q_paddr, "<>") != 0 &&
 			    sm_strncasecmp(e->e_from.q_paddr, "owner-", 6) != 0 &&
 			    (strlen(e->e_from.q_paddr) <= 8 ||
-			     sm_strcasecmp(&e->e_from.q_paddr[strlen(e->e_from.q_paddr) - 8],
-					   "-request") != 0))
+			     !SM_STRCASEEQ(&e->e_from.q_paddr[strlen(e->e_from.q_paddr) - 8],
+					   "-request")))
 			{
 				for (q = e->e_sendqueue; q != NULL;
 				     q = q->q_next)
@@ -380,7 +371,7 @@ dropenvelope(e, fulldrop, split)
 #if _FFR_NODELAYDSN_ON_HOLD
 					    && !bitnset(M_HOLD,
 							q->q_mailer->m_flags)
-#endif /* _FFR_NODELAYDSN_ON_HOLD */
+#endif
 					   )
 					{
 						if (msg_timeout ==
@@ -420,7 +411,7 @@ dropenvelope(e, fulldrop, split)
 				/* don't free, allocated from e_rpool */
 				e->e_message = sm_rpool_strdup_x(e->e_rpool,
 								 buf);
-				message(buf);
+				message("%s", buf);
 				e->e_flags |= EF_WARNING;
 			}
 			if (msg_timeout == MSG_WARN_BY)
@@ -446,8 +437,8 @@ dropenvelope(e, fulldrop, split)
 			failure_return, delay_return, success_return, queueit);
 
 	/*
-	**  If we had some fatal error, but no addresses are marked as
-	**  bad, mark them _all_ as bad.
+	**  If we had some fatal error, but no addresses are marked as bad,
+	**  mark all OK/VERIFIED addresses as bad (if QPINGONFAILURE).
 	*/
 
 	if (bitset(EF_FATALERRS, e->e_flags) && !failure_return)
@@ -455,8 +446,21 @@ dropenvelope(e, fulldrop, split)
 		for (q = e->e_sendqueue; q != NULL; q = q->q_next)
 		{
 			if ((QS_IS_OK(q->q_state) ||
-			     QS_IS_VERIFIED(q->q_state)) &&
-			    bitset(QPINGONFAILURE, q->q_flags))
+			     QS_IS_VERIFIED(q->q_state))
+			    && bitset(QPINGONFAILURE, q->q_flags)
+
+			/*
+			**  do not mark an address as bad if
+			**  - the address itself is stored in the queue
+			**  - the DeliveryMode requires queueing
+			**  - the envelope is queued
+			*/
+
+			    && !(bitset(QQUEUED, q->q_flags)
+				 && WILL_BE_QUEUED(e->e_sendmode)
+				 && bitset(EF_INQUEUE, e->e_flags)
+				)
+			   )
 			{
 				failure_return = true;
 				q->q_state = QS_BADADDR;
@@ -504,7 +508,7 @@ dropenvelope(e, fulldrop, split)
 	    e->e_class >= 0)
 	{
 		auto ADDRESS *rlist = NULL;
-		char pcopy[MAXNAME];
+		char pcopy[MAXNAME_I];
 
 		if (failure_return)
 		{
@@ -540,11 +544,7 @@ simpledrop:
 		}
 		if (!panic)
 		{
-			if (e->e_dfp != NULL)
-			{
-				(void) sm_io_close(e->e_dfp, SM_TIME_DEFAULT);
-				e->e_dfp = NULL;
-			}
+			SM_CLOSE_FP(e->e_dfp);
 			(void) xunlink(queuename(e, DATAFL_LETTER));
 		}
 		if (panic && QueueMode == QM_LOST)
@@ -571,7 +571,7 @@ simpledrop:
 	else if (queueit || !bitset(EF_INQUEUE, e->e_flags))
 	{
 		if (!split)
-			queueup(e, false, true);
+			queueup(e, QUP_FL_MSYNC);
 		else
 		{
 			ENVELOPE *oldsib;
@@ -592,10 +592,11 @@ simpledrop:
 				syserr("!dropenvelope(%s): cannot commit data file %s, uid=%ld",
 					e->e_id, queuename(e, DATAFL_LETTER),
 					(long) geteuid());
+				/* NOTREACHED */
 			}
 			for (ee = e->e_sibling; ee != NULL; ee = ee->e_sibling)
-				queueup(ee, false, true);
-			queueup(e, false, true);
+				queueup(ee, QUP_FL_MSYNC);
+			queueup(e, QUP_FL_MSYNC);
 
 			/* clean up */
 			for (ee = e->e_sibling; ee != NULL; ee = ee->e_sibling)
@@ -608,12 +609,7 @@ simpledrop:
 				unlockqueue(ee);
 
 				/* this envelope is marked unused */
-				if (ee->e_dfp != NULL)
-				{
-					(void) sm_io_close(ee->e_dfp,
-							   SM_TIME_DEFAULT);
-					ee->e_dfp = NULL;
-				}
+				SM_CLOSE_FP(ee->e_dfp);
 				ee->e_id = NULL;
 				ee->e_flags &= ~EF_HAS_DF;
 			}
@@ -628,11 +624,7 @@ simpledrop:
 	unlockqueue(e);
 
 	/* make sure that this envelope is marked unused */
-	if (e->e_dfp != NULL)
-	{
-		(void) sm_io_close(e->e_dfp, SM_TIME_DEFAULT);
-		e->e_dfp = NULL;
-	}
+	SM_CLOSE_FP(e->e_dfp);
 	e->e_id = NULL;
 	e->e_flags &= ~EF_HAS_DF;
 	if (panic)
@@ -677,11 +669,8 @@ clearenvelope(e, fullclear, rpool)
 	if (!fullclear)
 	{
 		/* clear out any file information */
-		if (e->e_xfp != NULL)
-			(void) sm_io_close(e->e_xfp, SM_TIME_DEFAULT);
-		if (e->e_dfp != NULL)
-			(void) sm_io_close(e->e_dfp, SM_TIME_DEFAULT);
-		e->e_xfp = e->e_dfp = NULL;
+		SM_CLOSE_FP(e->e_xfp);
+		SM_CLOSE_FP(e->e_dfp);
 	}
 
 	/*
@@ -737,7 +726,7 @@ clearenvelope(e, fullclear, rpool)
 	}
 #if _FFR_MILTER_ENHSC
 	e->e_enhsc[0] = '\0';
-#endif /* _FFR_MILTER_ENHSC */
+#endif
 }
 /*
 **  INITSYS -- initialize instantiation of system
@@ -878,7 +867,7 @@ settime(e)
 
 #ifndef O_APPEND
 # define O_APPEND	0
-#endif /* ! O_APPEND */
+#endif
 
 void
 openxscript(e)
@@ -892,7 +881,7 @@ openxscript(e)
 #if 0
 	if (e->e_lockfp == NULL && bitset(EF_INQUEUE, e->e_flags))
 		syserr("openxscript: job not locked");
-#endif /* 0 */
+#endif
 
 	p = queuename(e, XSCRPT_LETTER);
 	e->e_xfp = bfopen(p, FileMode, XscriptFileBufferSize,
@@ -904,7 +893,10 @@ openxscript(e)
 		e->e_xfp = sm_io_open(SmFtStdio, SM_TIME_DEFAULT,
 				      SM_PATH_DEVNULL, SM_IO_RDWR, NULL);
 		if (e->e_xfp == NULL)
+		{
 			syserr("!Can't open %s", SM_PATH_DEVNULL);
+			/* NOTREACHED */
+		}
 	}
 	(void) sm_io_setvbuf(e->e_xfp, SM_TIME_DEFAULT, NULL, SM_IO_LBF, 0);
 	if (tTd(46, 9))
@@ -936,9 +928,8 @@ closexscript(e)
 #if 0
 	if (e->e_lockfp == NULL)
 		syserr("closexscript: job not locked");
-#endif /* 0 */
-	(void) sm_io_close(e->e_xfp, SM_TIME_DEFAULT);
-	e->e_xfp = NULL;
+#endif
+	SM_CLOSE_FP(e->e_xfp);
 }
 /*
 **  SETSENDER -- set the person who this message is from
@@ -962,7 +953,7 @@ closexscript(e)
 **	ourselves.
 **
 **	Parameters:
-**		from -- the person we would like to believe this message
+**		from -- the person we would like to believe this message [i]
 **			is from, as specified on the command line.
 **		e -- the envelope in which we would like the sender set.
 **		delimptr -- if non-NULL, set to the location of the
@@ -990,7 +981,7 @@ setsender(from, e, delimptr, delimchar, internal)
 	register char **pvp;
 	char *realname = NULL;
 	char *bp;
-	char buf[MAXNAME + 2];
+	char buf[MAXNAME_I + 2];
 	char pvpbuf[PSBUFSIZE];
 	extern char *FullName;
 
@@ -1008,7 +999,7 @@ setsender(from, e, delimptr, delimchar, internal)
 	if (bitset(EF_QUEUERUN, e->e_flags) || OpMode == MD_SMTP ||
 	    OpMode == MD_ARPAFTP || OpMode == MD_DAEMON)
 		realname = from;
-	if (realname == NULL || realname[0] == '\0')
+	if (SM_IS_EMPTY(realname))
 		realname = username();
 
 	if (ConfigLevel < 2)
@@ -1031,7 +1022,7 @@ setsender(from, e, delimptr, delimchar, internal)
 		if (from != NULL && LogLevel > 2)
 		{
 			char *p;
-			char ebuf[MAXNAME * 2 + 2];
+			char ebuf[MAXNAME * 2 + 2]; /* EAI:ok? */
 
 			p = macvalue('_', e);
 			if (p == NULL)
@@ -1041,8 +1032,9 @@ setsender(from, e, delimptr, delimchar, internal)
 				if (host == NULL)
 					host = MyHostName;
 				(void) sm_snprintf(ebuf, sizeof(ebuf),
-						   "%.*s@%.*s", MAXNAME,
-						   realname, MAXNAME, host);
+						   "%.*s@%.*s",
+						   MAXNAME, realname, /* EAI: see above */
+						   MAXNAME, host); /* EAI: see above */
 				p = ebuf;
 			}
 			sm_syslog(LOG_NOTICE, e->e_id,
@@ -1061,6 +1053,7 @@ setsender(from, e, delimptr, delimchar, internal)
 			SuprErrs = true;
 		}
 		if (from == realname ||
+/* XXX realname must be [i] */
 		    parseaddr(from = realname,
 			      &e->e_from, RF_COPYALL|RF_SENDERADDR, ' ',
 			      NULL, e, false) == NULL)
@@ -1070,6 +1063,7 @@ setsender(from, e, delimptr, delimchar, internal)
 			SuprErrs = true;
 			expand("\201n", nbuf, sizeof(nbuf), e);
 			from = sm_rpool_strdup_x(e->e_rpool, nbuf);
+/* XXX from must be [i] */
 			if (parseaddr(from, &e->e_from, RF_COPYALL, ' ',
 				      NULL, e, false) == NULL &&
 			    parseaddr(from = "postmaster", &e->e_from,
@@ -1271,8 +1265,6 @@ static struct eflags	EnvelopeFlags[] =
 	{ "LOGSENDER",		EF_LOGSENDER	},
 	{ "NORECEIPT",		EF_NORECEIPT	},
 	{ "HAS8BIT",		EF_HAS8BIT	},
-	{ "NL_NOT_EOL",		EF_NL_NOT_EOL	},
-	{ "CRLF_NOT_EOL",	EF_CRLF_NOT_EOL	},
 	{ "RET_PARAM",		EF_RET_PARAM	},
 	{ "HAS_DF",		EF_HAS_DF	},
 	{ "IS_MIME",		EF_IS_MIME	},
@@ -1281,6 +1273,8 @@ static struct eflags	EnvelopeFlags[] =
 	{ "TOOBIG",		EF_TOOBIG	},
 	{ "SPLIT",		EF_SPLIT	},
 	{ "UNSAFE",		EF_UNSAFE	},
+	{ "TOODEEP",		EF_TOODEEP	},
+	{ "SECURE",		EF_SECURE	},
 	{ NULL,			0		}
 };
 
