@@ -162,8 +162,13 @@ mci_scan(savemci)
 		if ((mci->mci_lastuse + MciCacheTimeout <= now ||
 		     (mci->mci_mailer != NULL &&
 		      mci->mci_mailer->m_maxdeliveries > 0 &&
-		      mci->mci_deliveries + 1 >= mci->mci_mailer->m_maxdeliveries))&&
-		    mci != savemci)
+		      mci->mci_deliveries + 1 >= mci->mci_mailer->m_maxdeliveries)
+#if _FFR_LIMITS
+		     || (mci->mci_mailmax > 0 &&
+		      mci->mci_deliveries + 1 >= mci->mci_mailmax)
+#endif
+		     )
+		      && mci != savemci)
 		{
 			/* connection idle too long or too many deliveries */
 			bestmci = &MciCache[i];
@@ -255,6 +260,9 @@ mci_uncache(mcislot, doquit)
 	SM_FREE(mci->mci_status);
 	SM_FREE(mci->mci_rstatus);
 	SM_FREE(mci->mci_heloname);
+#if _FFR_LIMITS
+	SM_FREE(mci->mci_rcptdomain);
+#endif
 	mci_clear(mci);
 	if (mci->mci_rpool != NULL)
 	{
@@ -624,6 +632,7 @@ struct mcifbits
 };
 static struct mcifbits	MciFlags[] =
 {
+	{ MCIF_IO_ERR,		"MCIF_IO_ERR"	},
 	{ MCIF_OCC_INCR,	"OCC_INCR"	},
 	{ MCIF_CACHED,		"CACHED"	},
 	{ MCIF_ESMTP,		"ESMTP"		},
@@ -631,6 +640,9 @@ static struct mcifbits	MciFlags[] =
 	{ MCIF_SIZE,		"SIZE"		},
 	{ MCIF_8BITMIME,	"8BITMIME"	},
 	{ MCIF_7BIT,		"7BIT"		},
+#if _FFR_SAMEDOMAIN
+	{ MCIF_SAMEDOMAIN,	"SAMEDOMAIN"	},
+#endif
 	{ MCIF_INHEADER,	"INHEADER"	},
 	{ MCIF_CVT8TO7,		"CVT8TO7"	},
 	{ MCIF_DSN,		"DSN"		},
